"""
Hildon UI for Meabook
"""

import gtk
import hildon
import gobject
from gettext import gettext as _
from meabook.ui.ui import MeabookUI


def create_button(title, value):
    """Create HildonButton."""

    button = hildon.Button(gtk.HILDON_SIZE_AUTO | \
        gtk.HILDON_SIZE_FINGER_HEIGHT, \
        hildon.BUTTON_ARRANGEMENT_VERTICAL, title, value)
    button.set_style(hildon.BUTTON_STYLE_PICKER)
    button.set_alignment(0, 0, 0, 0)
    return button


def create_menu_button(title):
    return hildon.Button(gtk.HILDON_SIZE_AUTO, \
        hildon.BUTTON_ARRANGEMENT_HORIZONTAL, title)


class HildonMeabook(MeabookUI):
    def __init__(self, controller, renderer, config):
        MeabookUI.__init__(self, controller, renderer, config)
        self.handler = None
        self.window = hildon.StackableWindow()
        self.window.set_title(_('Meabook'))
        self.window.connect('destroy', self.exit)

        # create menu buttons
        self.menu = hildon.AppMenu()
        settings_button = create_menu_button(_('Settings'))
        about_button = create_menu_button(_('About'))
        import_button = create_menu_button(_('Import'))

        # create filter widgets and connect signals
        self.level1_filter = gtk.RadioButton(None, _('level1'))
        self.level2_filter = gtk.RadioButton(self.level1_filter, _('level2'))
        self.level3_filter = gtk.RadioButton(self.level2_filter, _('level3'))
        for filter_widget in [self.level1_filter, self.level2_filter, \
            self.level3_filter]:
            filter_widget.set_mode(False)
            filter_widget.connect('toggled', self.apply_filter_cb)

        # create items list
        self.selector = hildon.TouchSelector()

        # connect signals
        settings_button.connect('clicked', self.show_settings_dialog_cb)
        about_button.connect('clicked', self.show_about_dialog_cb)
        import_button.connect('clicked', self.show_import_dialog_cb)
        self.handler = self.selector.connect('changed', self.select_item_cb)

        # packing widgets
        self.window.add(self.selector)
        self.menu.add_filter(self.level1_filter)
        self.menu.add_filter(self.level2_filter)
        self.menu.add_filter(self.level3_filter)
        self.menu.append(settings_button)
        self.menu.append(import_button)
        self.menu.append(about_button)
        self.menu.show_all()
        self.window.set_app_menu(self.menu)
        self.window.show_all()

    def _unfreeze_ui(self):
        while gtk.events_pending():
            gtk.main_iteration(False)

    def _update_title(self, title):
        if title is not None:
            self.window.set_title(title)

    def _set_selector_content(self, selector, handler, items):
        """Updates selector content."""

        # temporary block handler
        selector.handler_block(handler)

        # setting new content
        # model: name, internal_name, type
        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT, \
            gobject.TYPE_STRING)
        for name, internal_name, item_type in items:
            model.append((name, internal_name, item_type))
        if selector.get_num_columns():
            selector.remove_column(0)
        selector.append_text_column(model, False)
        selector.unselect_all(0)

        # reconnect callback
        selector.handler_unblock(handler)

    def _show_items_dialog(self, title, items):
        """Creates dialog with items."""

        selector = hildon.TouchSelector()
        window = hildon.StackableWindow()
        window.set_title(title)
        window.add(selector)
        handler = selector.connect('changed', self.select_item_cb)
        self._set_selector_content(selector, handler, items)
        window.show_all()

    def _show_item_dialog(self, title, entry_id):
        """Shows detailed item information."""

        def show_settings_dialog(widget, parent, func1, func2, entry_id):
            dialog = ConfigurationDialog(self.controller, self.config)
            response = getattr(dialog, func1)(None, parent)
            if response == gtk.RESPONSE_OK:
                func2(parent, entry_id)

        def update_entry(window, entry_id):
            # create widgets
            entry = self.controller.get_item(entry_id)
            widgets_table = gtk.Table(rows=1, columns=1)
            info_box = gtk.VBox()
            image_box = gtk.VBox()
            pannable_area = hildon.PannableArea()
            pannable_area.set_property('mov-mode', hildon.MOVEMENT_MODE_BOTH)
            image = self.renderer.render_image(dict(entry))
            for fname, fvalue in entry:
                if fname == 'image':
                    continue
                button = self.renderer.render_button(_(fname) , fvalue, fname)
                info_box.pack_start(button, expand=False)
            # pack widgets
            image_box.pack_start(image, expand=False)
            widgets_table.attach(image_box, 0, 1, 0, 1, xoptions=gtk.SHRINK, \
                yoptions=gtk.FILL, xpadding=4, ypadding=8)
            pannable_area.add_with_viewport(info_box)
            widgets_table.attach(pannable_area, 1, 2, 0, 1, ypadding=8)
            child = window.get_child()
            if child:
                child.destroy()
            window.add(widgets_table)
            widgets_table.show_all()


        # create widgets
        window = hildon.StackableWindow()
        window.set_title(title)
        menu = hildon.AppMenu()
        fields_button = create_menu_button(_('Fields to show'))
        order_button = create_menu_button(_('Fields order'))
        fields_button.connect('clicked', show_settings_dialog, window, \
            'show_fields_settings_cb', update_entry, entry_id)
        order_button.connect('clicked', show_settings_dialog, window, \
            'show_order_settings_cb', update_entry, entry_id)
        menu.append(fields_button)
        menu.append(order_button)

        update_entry(window, entry_id)
        window.set_app_menu(menu)
        menu.show_all()
        window.show_all()


    # Implementation of Base UI interface
    def start(self):
        self.apply_filter_cb(self.level1_filter)
        gtk.main()

    def exit(self, event):
        gtk.main_quit()
        self.controller.stop()

    def create_information_dialog(self, title, message):
        dialog = hildon.Dialog()
        dialog.set_title(title)
        label = gtk.Label(message)
        dialog.vbox.add(label)
        dialog.vbox.show_all()
        dialog.run()
        dialog.destroy()

    def create_about_dialog(self):
        dialog = hildon.Dialog()
        dialog.set_title(_('About'))
        label = gtk.Label('\nMeabook v0.1\n')
        dialog.vbox.add(label)
        dialog.vbox.show_all()
        dialog.run()
        dialog.destroy()

    def create_import_dialog(self):
        chooser = gobject.new(hildon.FileChooserDialog, \
            action=gtk.FILE_CHOOSER_ACTION_OPEN)
        chooser.set_property('show-files', True)
        chooser.run()
        path = chooser.get_filename()
        chooser.destroy()
        return path

    def create_progress_dialog(self, title=None):
        self._update_title(title)
        self.selector.hide()
        hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
        self._unfreeze_ui()

    def create_configuration_dialog(self, controller, config):
        dialog = ConfigurationDialog(controller, config)
        dialog.run()


    # Hildon UI callbacks
    def show_about_dialog_cb(self, widget):
        """Shows About Dialog."""

        self.controller.show_about_dialog()

    def show_import_dialog_cb(self, widget):
        """Shows Import Dialog."""

        if self.controller.show_import_dialog():
            self.apply_filter_cb(self.level1_filter)
            self.selector.show_all()
        hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)

    def show_settings_dialog_cb(self, widget):
        """Shows Config dialog."""

        self.controller.show_configuration_dialog()

    def apply_filter_cb(self, widget):
        """Updates toplevel selector with different level items."""

        if not widget.get_active():
            return

        self._update_title(' - '.join([_('Meabook'), widget.get_label()]))

        if widget == self.level1_filter:
            self._set_selector_content(self.selector, self.handler, \
                self.controller.get_all_folders())
        elif widget == self.level2_filter:
            self._set_selector_content(self.selector, self.handler, \
                self.controller.get_all_subfolders())
        else:
            self._set_selector_content(self.selector, self.handler, \
                self.controller.get_all_files())

    def select_item_cb(self, widget, column):
        """
        Emits when changes selector content.
        Opens new StackableWindow with new content.
        """

        selected_item_index = widget.get_active(0)
        _iter = widget.get_model(0)[selected_item_index]
        if _iter[2] == 'd':
            self._show_items_dialog(_iter[0], self.controller.get_items(\
                _iter[1]))
        else:
            self._show_item_dialog(_iter[0], _iter[1])




class ConfigurationDialog:
    """Configuration dialog"""

    def __init__(self, controller, config):
        self.config = config
        self.controller = controller

    def run(self):
        dialog = hildon.Dialog()
        dialog.set_title(_('Settings'))

        button_order = create_button(_('Fields order'), self._update_value( \
            None, self.config.get_order()))
        button_order.connect('clicked', self.show_order_settings_cb, dialog)
        button_fields = create_button(_('Fields to show'), self._update_value( \
            None, self.config.get_fields()))
        button_fields.connect('clicked', self.show_fields_settings_cb, dialog)

        dialog.vbox.pack_start(button_fields, expand=False)
        dialog.vbox.pack_start(button_order, expand=False)
        dialog.vbox.show_all()
        dialog.run()
        dialog.destroy()

    def _update_value(self, widget, fields):
        """Updates widget title."""

        value = ', '.join([_(field) for field in fields])
        if widget is not None:
            widget.set_value(value)
        return value

    def show_fields_settings_cb(self, widget, parent):
        """Shows dialog for selecting fields to show."""

        dialog = hildon.PickerDialog(parent)
        dialog.set_title(_('Fields to show'))
        selector = hildon.TouchSelector(text=True)

        fields = self.controller.get_fields()
        # fill items list
        for field in fields:
            selector.append_text(_(field))

        selector.set_column_selection_mode( \
            hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
        selector.unselect_all(0)

        # mark necessary fields
        fields_to_select = self.config.get_fields()
        model = selector.get_model(0)
        for index, field in enumerate(fields):
            if field in fields_to_select:
                selector.select_iter(0, model.get_iter(index), False)

        dialog.set_selector(selector)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            model = selector.get_model(0)
            selected_item_indexes = [index for index in [item[0] for item \
                in selector.get_selected_rows(0)]]
            selected_fields = [fields[index] for index in selected_item_indexes]
            self.config.set_fields(selected_fields)
            self._update_value(widget, selected_fields)
        dialog.destroy()
        return response

    def show_order_settings_cb(self, widget, parent):
        """Shows dialog for setting fields order."""


        def show_fields_chooser(widget, parent):
            """Shows dialog to select field from fields list."""

            dialog = hildon.PickerDialog(parent)
            dialog.set_title(_('Fields'))
            selector = hildon.TouchSelector(text=True)
            dialog.set_selector(selector)
            selector.set_column_selection_mode( \
                hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
            # fill fields list
            for field in self.controller.get_fields():
                selector.append_text(_(field))
            dialog.run()
            widget.set_value(selector.get_current_text())
            dialog.destroy()


        dialog = hildon.Dialog()
        dialog.set_title(_('Fields order'))
        pannable_area = hildon.PannableArea()
        pannable_area.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)

        vbox = gtk.VBox()
        fields = self.config.get_order()
        for index, field in enumerate(fields):
            button = create_button(' '.join([_('Position'), str(index)]), \
                _(field))
            button.connect('clicked', show_fields_chooser, dialog)
            vbox.pack_start(button, expand=False)
        pannable_area.add_with_viewport(vbox)
        dialog.add_button(_('Done'), gtk.RESPONSE_OK)
        dialog.vbox.pack_start(pannable_area)
        dialog.vbox.show_all()
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            fields_dict = dict([(_(field), field) for field in fields])
            new_ordered_fields = [fields_dict[button.get_value()] for button \
                in vbox.get_children()]
            self.config.set_order(new_ordered_fields)
            self._update_value(widget, new_ordered_fields)
        dialog.destroy()
        return response
